@@ -19,7 +19,16 @@ squirrel = Squirrel()
 
 dts = '%Y-%m-%dT%H:%M:%S'
 
-#________________________________________________________________
+#====================================================================================================
+node_types = {
+	'P': 'product',
+	'E': 'entity',
+	'C': 'classification',
+	'A' : 'asset',
+	'a' : 'attribute',
+}
+
+#====================================================================================================
 class Silencer(object):
 	def write(self, *args, **kwargs):
 		pass
@@ -29,7 +38,7 @@ class Silencer(object):
 		pass
 
 
-#________________________________________________________________
+#====================================================================================================
 @args.command(name='step')
 class STEP(object):
 	'''
@@ -69,6 +78,7 @@ class STEP(object):
 	@args.property(short='X')
 	def xslt(self): return
 
+	#________________________________________________________________________________________________
 	def __init__(self, asXML=None, verbose=None, output=None, silent=True, hostname=None, username=None, context=None, workspace=None):
 		if asXML: self.asXML = asXML
 		if verbose: self.verbose = verbose
@@ -86,6 +96,7 @@ class STEP(object):
 		self.path = 'restapi' if self.asXML else 'restapiv2'
 
 		
+	#________________________________________________________________________________________________
 	def process(self, path, params=None, kwargs=dict()):
 		'''
 		shared get operation
@@ -98,17 +109,20 @@ class STEP(object):
 		}
 		if self.verbose:
 			json.dump(dict(url=url, headers=self.headers, params=params), sys.stderr, indent=4)
-		result = requests.get(url=url, auth=auth, headers=self.headers, params=params, **kwargs)
-		if result.status_code != 200 or self.verbose:
-			sys.stderr.write('%s: %s\n'%(result, result.text))
+		result = None
+		with requests.get(url=url, auth=auth, headers=self.headers, params=params, **kwargs) as result:
+			if result.status_code != 200 or self.verbose:
+				sys.stderr.write('%s: %s\n'%(result, result.text))
 		return result
 
 	
+	#________________________________________________________________________________________________
 	def get(self, path, params=None):
 		result = self.process(path, params, kwargs=dict())
 		return self.export(result.text)
 
 	
+	#________________________________________________________________________________________________
 	def put(self, path, body=None, params=None, headers=None):
 		url = '%s/%s/%s'%(self.hostname, self.path, path)
 		auth= (self.username, self.password)
@@ -122,12 +136,14 @@ class STEP(object):
 			headers['Content-Type'] = headers['Accept']
 		if self.verbose:
 			json.dump(dict(url=url, headers=headers, params=params), sys.stderr, indent=4)
-		result = requests.put(url=url, auth=auth, headers=headers, params=params, data=body)
-		if result.status_code not in [200,201] or self.verbose:
-			sys.stderr.write('%s: %s\n'%(result, result.text))
+		result = None
+		with requests.put(url=url, auth=auth, headers=headers, params=params, data=body) as result:
+			if result.status_code not in [200,201] or self.verbose:
+				sys.stderr.write('%s: %s\n'%(result, result.text))
 		return result.text
 
 	
+	#________________________________________________________________________________________________
 	def post(self, path, body=None, params=None, headers=None):
 		url = '%s/%s/%s'%(self.hostname, self.path, path)
 		auth= (self.username, self.password)
@@ -141,12 +157,14 @@ class STEP(object):
 			headers['Content-Type'] = headers['Accept']
 		if self.verbose:
 			json.dump(dict(url=url, headers=headers, params=params), sys.stderr, indent=4)
-		result = requests.post(url=url, auth=auth, headers=headers, params=params, data=body)
-		if result.status_code not in [200,201] or self.verbose:
-			sys.stderr.write('%s: %s\n'%(result, result.text))
+		result = None
+		with requests.post(url=url, auth=auth, headers=headers, params=params, data=body) as result:
+			if result.status_code not in [200,201] or self.verbose:
+				sys.stderr.write('%s: %s\n'%(result, result.text))
 		return result.text
 
 
+	#________________________________________________________________________________________________
 	def delete(self, path, body=None, params=None, headers=None):
 		url = '%s/%s/%s'%(self.hostname, self.path, path)
 		auth= (self.username, self.password)
@@ -160,12 +178,14 @@ class STEP(object):
 			headers['Content-Type'] = headers['Accept']
 		if self.verbose:
 			json.dump(dict(url=url, headers=headers, params=params), sys.stderr, indent=4)
-		result = requests.delete(url=url, auth=auth, headers=headers, params=params, data=body)
-		if result.status_code != 200 or self.verbose:
-			sys.stderr.write('%s: %s\n'%(result, result.text))
+		result = None
+		with requests.delete(url=url, auth=auth, headers=headers, params=params, data=body) as result:
+			if result.status_code != 200 or self.verbose:
+				sys.stderr.write('%s: %s\n'%(result, result.text))
 		return result.text
 
 	
+	#________________________________________________________________________________________________
 	def file(self, path, destination):
 		params={
 			"context" : self.context,
@@ -180,6 +200,7 @@ class STEP(object):
 		return
 
 
+	#________________________________________________________________________________________________
 	def export(self, result):
 		if not result: return
 		result = result if self.asXML else json.loads(result)
@@ -206,7 +227,7 @@ class STEP(object):
 		return result
 			
 
-#________________________________________________________________
+#====================================================================================================
 @args.command(name='assets')
 class Assets(STEP):
 	'''
@@ -215,10 +236,12 @@ class Assets(STEP):
 	
 	base = 'assets'
 	
+	#________________________________________________________________________________________________
 	def __init__(self, asXML=None, verbose=None, output=None, silent=True, hostname=None, username=None, context=None, workspace=None):
 		super().__init__(asXML=asXML, verbose=verbose, output=output, silent=silent, hostname=hostname, username=username, context=context, workspace=workspace)
 		
 
+	#________________________________________________________________________________________________
 	@args.operation
 	def get(self, id):
 		'''
@@ -227,6 +250,7 @@ class Assets(STEP):
 		return super().get('%s/%s'%(self.base,id))
 
 
+	#________________________________________________________________________________________________
 	@args.operation
 	@args.parameter(name='output', short='o', help='where to store the content, defautls to stdout')
 	def content(self, id, output=None):
@@ -238,11 +262,13 @@ class Assets(STEP):
 		super().file(path=path, destination=output)
 
 
+	#________________________________________________________________________________________________
 	@args.operation
 	def upload(self, id, file):
 		pass
 
 
+	#________________________________________________________________________________________________
 	@args.operation
 	def update(self, id, file):
 		with open(file) as input:
@@ -251,7 +277,7 @@ class Assets(STEP):
 			return super().put('%s/%s/content'%(self.base, id), body=body, headers=headers)
 	
 
-#________________________________________________________________
+#====================================================================================================
 @args.command(name='processes')
 class Processes(STEP):
 	'''
@@ -259,10 +285,12 @@ class Processes(STEP):
 	'''
 	base = 'backgroundprocesses'
 
+	#________________________________________________________________________________________________
 	def __init__(self, asXML=None, verbose=None, output=None, silent=True, hostname=None, username=None, context=None, workspace=None):
 		super().__init__(asXML=asXML, verbose=verbose, output=output, silent=silent, hostname=hostname, username=username, context=context, workspace=workspace)
 		
 
+	#________________________________________________________________________________________________
 	@args.operation
 	def list(self):
 		'''
@@ -271,6 +299,7 @@ class Processes(STEP):
 		return super().get(self.base)
 
 	
+	#________________________________________________________________________________________________
 	@args.operation
 	def get(self, id):
 		'''
@@ -282,6 +311,7 @@ class Processes(STEP):
 			return super().get('background-processes/%s'%id)
 
 		
+	#________________________________________________________________________________________________
 	@args.operation(help='get error report, JSON only !')
 	def report(self, id):
 		'''
@@ -290,6 +320,7 @@ class Processes(STEP):
 		return super().get('background-processes/%s/execution-report'%id)
 
 	
+	#________________________________________________________________________________________________
 	@args.operation(help='get attachments, JSON only !')
 	def attachments(self, id):
 		'''
@@ -298,6 +329,7 @@ class Processes(STEP):
 		return super().get('background-processes/%s/attachments'%id)
 
 	
+	#________________________________________________________________________________________________
 	@args.operation(help='get attachment metadata, JSON only !')
 	def attachment_metadata(self, id, attachmentId):
 		'''
@@ -305,6 +337,8 @@ class Processes(STEP):
 		'''
 		return super().get('background-processes/%s/attachments/%s'%(id,attachmentId))
 
+
+	#________________________________________________________________________________________________
 	@args.operation(help='get attachment content, JSON only !')
 	def attachment_content(self, id, attachmentId):
 		'''
@@ -313,7 +347,7 @@ class Processes(STEP):
 		return super().get('background-processes/%s/attachments/%s/content'%(id,attachmentId))
 
 	
-#________________________________________________________________
+#====================================================================================================
 @args.command(name='instances')
 class Instances(STEP):
 	'''
@@ -321,10 +355,12 @@ class Instances(STEP):
 	'''
 	base = 'bgpinstance'
 
+	#________________________________________________________________________________________________
 	def __init__(self, asXML=None, verbose=None, output=None, silent=True, hostname=None, username=None, context=None, workspace=None):
 		super().__init__(asXML=asXML, verbose=verbose, output=output, silent=silent, hostname=hostname, username=username, context=context, workspace=workspace)
 		
 
+	#________________________________________________________________________________________________
 	@args.operation
 	def list(self):
 		'''
@@ -333,6 +369,7 @@ class Instances(STEP):
 		return super().get(self.base)
 	
 
+	#________________________________________________________________________________________________
 	@args.operation
 	def report(self, id):
 		'''
@@ -341,6 +378,7 @@ class Instances(STEP):
 		return super().get('%s/%s/executionreport'%(self.base,id))
 
 
+	#________________________________________________________________________________________________
 	@args.operation
 	def status(self, id):
 		'''
@@ -349,6 +387,7 @@ class Instances(STEP):
 		return super().get('%s/%s/status'%(self.base,id))
 
 
+	#________________________________________________________________________________________________
 	@args.operation
 	def attachment(self, id, attchmentID):
 		'''
@@ -357,16 +396,18 @@ class Instances(STEP):
 		return super().get('%s/%s/attachment/%s'%(self.base,id,attachmentID))
 
 
-#________________________________________________________________
+#====================================================================================================
 @args.command(name='objects')
 class ObjectsByKey(STEP):
 
 	base = 'objectsbykey'
 	
+	#________________________________________________________________________________________________
 	def __init__(self, asXML=None, verbose=None, output=None, silent=True, hostname=None, username=None, context=None, workspace=None):
 		super().__init__(asXML=asXML, verbose=verbose, output=output, silent=silent, hostname=hostname, username=username, context=context, workspace=workspace)
 	
 
+	#________________________________________________________________________________________________
 	@args.operation(help='get object by keyid and key')
 	@args.parameter(name='keyid', help='the ID of object type')
 	@args.parameter(name='key', help='the key of the object')
@@ -374,22 +415,25 @@ class ObjectsByKey(STEP):
 		return super().get('%s/%s/%s'%(self.base,keyid,key))
 
 
-#________________________________________________________________
+#====================================================================================================
 @args.command(name='products')
 class Products(STEP):
 
 	base = 'products'
 
+	#________________________________________________________________________________________________
 	def __init__(self, asXML=None, verbose=None, output=None, silent=True, hostname=None, username=None, context=None, workspace=None):
 		super().__init__(asXML=asXML, verbose=verbose, output=output, silent=silent, hostname=hostname, username=username, context=context, workspace=workspace)
 		
 
+	#________________________________________________________________________________________________
 	@args.operation(help='list the products')
 	@args.parameter(name='root',short='r')
 	def list(self,root='ProductsRoot'):
 		return super().get('%s/%s/children'%(self.base,root))
 
 
+	#________________________________________________________________________________________________
 	@args.operation(help='get product by id')
 	@args.parameter(name='id', help='the ID of product')
 	@args.parameter(name='save', short='s', flag=True, help='save to local /restapi')
@@ -401,6 +445,7 @@ class Products(STEP):
 		return result
 
 	
+	#________________________________________________________________________________________________
 	@args.operation(help='create a new product')
 	@args.parameter(name='name', short='n')
 	@args.parameter(name='values', short='a', nargs='*', metavar='attr=value')
@@ -417,10 +462,10 @@ class Products(STEP):
 		if self.verbose:
 			json.dump(body, sys.stderr, indent=4)
 		result = super().post('%s'%self.base, body=json.dumps(body))
-		result = json.loads(result.text)
-		return result
+		return json.loads(result)
 
 			
+	#________________________________________________________________________________________________
 	@args.operation(help='delete product by id')
 	@args.parameter(name='id', help='the ID of product')
 	def delete(self, id):
@@ -428,12 +473,14 @@ class Products(STEP):
 		return result
 
 
+	#________________________________________________________________________________________________
 	@args.operation(help='get children of product by id')
 	@args.parameter(name='id', help='the ID of product')
 	def children(self, id):
 		return super().get('%s/%s/children'%(self.base,id))
 
 
+	#________________________________________________________________________________________________
 	@args.operation(help='get values of product by id')
 	@args.parameter(name='id', help='the ID of product')
 	@args.parameter(name='attributeID', help='the ID of product')
@@ -441,27 +488,32 @@ class Products(STEP):
 		return super().get('%s/%s/values/%s'%(self.base, id, attributeID))
 
 
+	#________________________________________________________________________________________________
 	@args.operation(help='get references of product by id')
 	@args.parameter(name='id', help='the ID of product')
 	@args.parameter(name='referenceID', help='the ID of reference')
 	def references(self, id, referenceID):
 		return super().get('%s/%s/references/%s'%(self.base, id, referenceID))
 	
+
+	#________________________________________________________________________________________________
 	@args.operation(help='get references of product by id')
 	@args.parameter(name='id', help='the ID of product')
 	@args.parameter(name='referenceID', help='the ID of reference')
 	@args.parameter(name='targetID', help='the ID of target')
+	@args.parameter(name='targetType', short=True, oneof=node_types, help='target type')
 	@args.parameter(name='values', short='m', metavar='attr=value', nargs='*', help='metadata attribute on reference')
 	@args.parameter(name='asid', short='i', flag=True, help='reference valus as lov id')
 	@args.parameter(name='overwrite', short='w', flag=True, help='allow overwrite')
-	def reference(self, id, referenceID, targetID, values=[], asid=False, overwrite=False):
+	@args.parameter(name='remove', short='r', flag=True, help='allow overwrite')
+	def reference(self, id, referenceID, targetID, targetType='product', values=[], asid=False, overwrite=False, remove=False):
 		headers={
 			"accept": "application/json",
 			"Content-Type": "application/json",
 		}
 		payload={
 			"target": targetID,
-			"targetType": "product",
+			"targetType": node_types[targetType],
 			"values": {
 			}
 		}
@@ -476,16 +528,28 @@ class Products(STEP):
 					"unit": None
 				}
 			}
+
 		params = {
 			"context" : self.context,
 			"workspace": self.workspace,
-			"allow-overwrite" : overwrite
 		}
-		return super().put('%s/%s/references/%s/%s'%(
+
+		if remove:
+			method = super().delete
+		else:
+			params["allow-overwrite"] = overwrite
+			method = super().put
+
+		result = method('%s/%s/references/%s/%s'%(
 			self.base, id, referenceID, targetID), body=json.dumps(payload), params=params, headers=headers
 		)
+		try:
+			return json.loads(result)
+		except:
+			return result
 
 
+	#________________________________________________________________________________________________
 	@args.operation(help='set values of product by id')
 	@args.parameter(name='id', help='the ID of product')
 	@args.parameter(name='attributeID', help='the ID of product')
@@ -500,59 +564,69 @@ class Products(STEP):
 				"value":value
 			}
 		}
-		return super().put('%s/%s/values/%s'%(
+		result = super().put('%s/%s/values/%s'%(
 			self.base, id, attributeID), body=json.dumps(payload), headers=headers
 		)
+		return json.loads(result)
 
 
+	#________________________________________________________________________________________________
 	@args.operation(help='get tables of product by id')
 	@args.parameter(name='id', help='the ID of product')
 	def tables(self, id):
 		return super().get('%s/%s/tables'%(self.base,id))
 
 
+	#________________________________________________________________________________________________
 	@args.operation(help='search for a product')
 	@args.parameter(name='expression', help='the search expression * allowed')
 	def search(self, expression):
 		return super().get('basicsearch/%s'%expression)
 
 
-#________________________________________________________________
+#====================================================================================================
 @args.command(name='entities')
 class Entities(STEP):
 
 	base = 'entities'
 
+	#________________________________________________________________________________________________
 	def __init__(self, asXML=None, verbose=None, output=None, silent=True, hostname=None, username=None, context=None, workspace=None):
 		super().__init__(asXML=asXML, verbose=verbose, output=output, silent=silent, hostname=hostname, username=username, context=context, workspace=workspace)
 
 		
+	#________________________________________________________________________________________________
 	@args.property(short='r', default='Entity hierarchy root')
 	def root(self): return
 	
+	#________________________________________________________________________________________________
 	@args.operation(help='list of children of entity hierarchy root')
 	def list(self):
 		return super().get('%s/%s/children'%(self.base,self.root))
 
 
+	#________________________________________________________________________________________________
 	@args.operation(help='get entity by id')
 	@args.parameter(name='id', help='the ID of entity')
 	def get(self, id):
 		return super().get('%s/%s'%(self.base,id))
 
 
+	#________________________________________________________________________________________________
 	@args.operation(help='get children of entity by id')
 	@args.parameter(name='id', help='the ID of entity')
 	def children(self, id):
 		return super().get('%s/%s/children'%(self.base,id))
 
 
+	#________________________________________________________________________________________________
 	@args.operation(help='get values of entity by id')
 	@args.parameter(name='id', help='the ID of entity')
 	def values(self, id):
 		return super().get('%s/%s/values'%(self.base,id))
 
 
+	#________________________________________________________________________________________________
 	@args.operation(help='update values of entity by id')
 	@args.parameter(name='id', help='the ID of entity')
 	@args.parameter(name='name', help='attribute ID')
@@ -563,6 +637,7 @@ class Entities(STEP):
 		return super().put('%s/%s/values/%s'%(self.base, id, name), body=body, headers=headers)
 
 	
+	#________________________________________________________________________________________________
 	@args.operation(help='search for entities')
 	@args.parameter(name='below_id', help='the ID of entity to search within')
 	@args.parameter(name='conditionType', choices=['name'], default='name', help='the type of confidition')
@@ -590,120 +665,160 @@ class Entities(STEP):
 		return super().post('%s/search'%(self.base), body=body, headers=headers)
 		
 
-#________________________________________________________________
+#====================================================================================================
 @args.command(name='classifications')
 class Classifications(STEP):
 
 	base = 'classifications'
 
+	#________________________________________________________________________________________________
 	def __init__(self, asXML=None, verbose=None, output=None, silent=True, hostname=None, username=None, context=None, workspace=None):
 		super().__init__(asXML=asXML, verbose=verbose, output=output, silent=silent, hostname=hostname, username=username, context=context, workspace=workspace)
 
 		
+	#________________________________________________________________________________________________
 	@args.property(short='r', default='Metcash_Root_Metcash')
 	def root(self): return
 	
+	#________________________________________________________________________________________________
 	@args.operation(help='list of children of classification hierarchy root')
 	def list(self):
 		return super().get('%s/%s/children'%(self.base,self.root))
 
 
+	#________________________________________________________________________________________________
 	@args.operation(help='get classification by id')
 	@args.parameter(name='id', help='the ID of classification')
 	def get(self, id):
 		return super().get('%s/%s'%(self.base,id))
 
 
+	#________________________________________________________________________________________________
+	@args.operation(help='create a new classification')
+	@args.parameter(name='name', short='n')
+	@args.parameter(name='values', short='a', nargs='*', metavar='attr=value')
+	def create(self, parent, objectType, name=None, values=[]):
+		body=dict(
+			parent=parent,
+			objectType=objectType,
+			name=name,
+			values=dict()
+		)
+		for nvp in values:
+			(attr,value) = tuple(nvp.split('='))
+			body['values'][attr] = dict(value=dict(value=value))
+		if self.verbose:
+			json.dump(body, sys.stderr, indent=4)
+		result = super().post('%s'%self.base, body=json.dumps(body))
+		return json.loads(result)
+		
+
+	#________________________________________________________________________________________________
 	@args.operation(help='get children of classification by id')
 	@args.parameter(name='id', help='the ID of classification')
 	def references(self, parent_id, reference_id):
 		return super().get('%s/%s/incoming-references/%s'%(self.base, parent_id, reference_id))
 
 	
+	#________________________________________________________________________________________________
 	@args.operation(help='get children of classification by id')
 	@args.parameter(name='id', help='the ID of classification')
 	def assets(self, id):
 		return super().get('%s/%s/assets'%(self.base, id))
 
 	
+	#________________________________________________________________________________________________
 	@args.operation(help='get children of classification by id')
 	@args.parameter(name='id', help='the ID of classification')
 	def children(self, id):
 		return super().get('%s/%s/children'%(self.base,id))
 
 	
+	#________________________________________________________________________________________________
 	@args.operation(help='get values of classification by id')
 	@args.parameter(name='id', help='the ID of classification')
 	def values(self, id):
 		return super().get('%s/%s/values'%(self.base,id))
 	
 
-#________________________________________________________________
+#====================================================================================================
 @args.command(name='endpoints')
 class Endpoints(STEP):
 
 	base = 'integrationendpoints'
 
+	#________________________________________________________________________________________________
 	def __init__(self, asXML=None, verbose=None, output=None, silent=True, hostname=None, username=None, context=None, workspace=None):
 		super().__init__(asXML=asXML, verbose=verbose, output=output, silent=silent, hostname=hostname, username=username, context=context, workspace=workspace)
 		
 
+	#________________________________________________________________________________________________
 	@args.operation(help='get a list of endpoints')
 	def list(self):
 		return super().get('%s'%self.base)
 
 	
+	#________________________________________________________________________________________________
 	@args.operation(help='get a list of inbound endpoints, JSON only !')
 	def list_inbound(self):
 		return super().get('inbound-integration-endpoints')
 	
 
+	#________________________________________________________________________________________________
 	@args.operation(help='get the status of an inbound endpoint, JSON only !')
 	def status_inbound(self, id):
 		return super().get('inbound-integration-endpoints/%s/status'%id)
 	
 
+	#________________________________________________________________________________________________
 	@args.operation(help='get the background processes of an inbound endpoint, JSON only !')
 	def processes_inbound(self, id):
 		return super().get('inbound-integration-endpoints/%s/worker-processes'%id)
 	
 
+	#________________________________________________________________________________________________
 	@args.operation(help='get a list of outbound endpoints, JSON only !')
 	def list_outbound(self):
 		return super().get('outbound-integration-endpoints')
 	
 
+	#________________________________________________________________________________________________
 	@args.operation(help='get the status of an outbound endpoint, JSON only !')
 	def status_outbound(self, id):
 		return super().get('outbound-integration-endpoints/%s/status'%id)
 	
 
+	#________________________________________________________________________________________________
 	@args.operation(help='get the background processes of an outbound endpoint, JSON only !')
 	def processes_outbound(self, id):
 		return super().get('outbound-integration-endpoints/%s/worker-processes'%id)
 	
 
+	#________________________________________________________________________________________________
 	@args.operation(help='get the logs')
 	def log(self, id):
 		return super().get('%s/%s/log'%(self.base, id))
 		
 
+	#________________________________________________________________________________________________
 	@args.operation(help='get errors')
 	def errors(self, id):
 		return super().get('%s/%s/errorexcerpts'%(self.base, id))
 		
 
+	#________________________________________________________________________________________________
 	@args.operation(help='get background processes')
 	def processes(self, id):
 		return super().get('%s/%s/backgroundprocesses'%(self.base, id))
 	
 
+	#________________________________________________________________________________________________
 	@args.operation(help='invoke an endpoint')
 	def invoke(self, id):
 		return super().put('%s/%s/invoke'%(self.base, id))
 											   
 
-#________________________________________________________________
+#====================================================================================================
 @args.command(name='imports')
 class Imports(STEP):
 	'''
@@ -712,10 +827,12 @@ class Imports(STEP):
 	
 	base = 'import'
 	
+	#________________________________________________________________________________________________
 	def __init__(self, asXML=None, verbose=None, output=None, silent=True, hostname=None, username=None, context=None, workspace=None):
 		super().__init__(asXML=asXML, verbose=verbose, output=output, silent=silent, hostname=hostname, username=username, context=context, workspace=workspace)
 		
 
+	#________________________________________________________________________________________________
 	@args.operation(name='import')
 	@args.parameter(name='id', help='importConfigurationId')
 	@args.parameter(name='process', short='p', help='process BGP description')
@@ -733,10 +850,10 @@ class Imports(STEP):
 			body = input.read()
 			headers = { 'Content-Type' : 'application/octet-stream' }
 			result = super().post('%s/%s'%(self.base, id), body=body, headers=headers, params=params)
-			return result.json()
+			return result
 		
 
-#________________________________________________________________
+#====================================================================================================
 @args.command(name='exports')
 class Exports(STEP):
 	'''
@@ -745,10 +862,12 @@ class Exports(STEP):
 	
 	base = 'export'
 	
+	#________________________________________________________________________________________________
 	def __init__(self, asXML=None, verbose=None, output=None, silent=True, hostname=None, username=None, context=None, workspace=None):
 		super().__init__(asXML=asXML, verbose=verbose, output=output, silent=silent, hostname=hostname, username=username, context=context, workspace=workspace)
 		
 
+	#________________________________________________________________________________________________
 	@args.operation(name='export')
 	@args.parameter(name='id', help='exportConfigurationId')
 	@args.parameter(name='use_context', short='c', flag=True, help='useRequestContextWorkspace')
@@ -770,37 +889,34 @@ class Exports(STEP):
 		}
 		headers = { 'Content-Type' : 'application/json' }
 		result = super().post('%s/%s'%(self.base, id), body=json.dumps(body), headers=headers, params=params)
-		return result.json()
+		return json.loads(result)
 
 
-#________________________________________________________________
-node_types = {
-	'P': 'product',
-	'E': 'entity',
-	'C': 'classification',
-	'A' : 'asset',
-	'a' : 'attribute',
-}
-
-#________________________________________________________________
+#====================================================================================================
 @args.command(name='workflow')
 class Workflow(STEP):
 
 	base = 'workflows'
 
+	#________________________________________________________________________________________________
 	def __init__(self, asXML=None, verbose=None, output=None, silent=True, hostname=None, username=None, context=None, workspace=None):
 		super().__init__(asXML=asXML, verbose=verbose, output=output, silent=silent, hostname=hostname, username=username, context=context, workspace=workspace)
 		
 
+	#________________________________________________________________________________________________
 	@args.operation(help='get a list of workflows')
 	def list(self):
 		return super().get('%s'%self.base)				
 
+
+	#________________________________________________________________________________________________
 	@args.operation(help='get workflow by id')
 	@args.parameter(name='id', help='the ID of workflow definition')
 	def get(self, id):
 		return super().get('%s/%s'%(self.base,id))
 
+
+	#________________________________________________________________________________________________
 	@args.operation(help='instantiate a workflow instance')
 	@args.parameter(name='workflow_id', help='the ID of workflow')
 	@args.parameter(name='node_id', help='the node ID')
@@ -833,6 +949,8 @@ class Workflow(STEP):
 		instance = base64.b64decode(response['id']).decode('UTF-8')
 		return json.loads(instance)
 		
+
+	#________________________________________________________________________________________________
 	@args.operation(help='terminate a workflow instance')
 	@args.parameter(name='workflow_id', help='the ID of workflow')
 	@args.parameter(name='instance_id', help='the ID of instance')
@@ -843,16 +961,18 @@ class Workflow(STEP):
 		return super().delete('%s/%s/instances/%s'%(self.base, workflow_id, instance_id))		
 		
 
-#________________________________________________________________
+#====================================================================================================
 @args.command(name='tasks')
 class Task(STEP):
 
 	base = 'workflow-tasks'
 
+	#________________________________________________________________________________________________
 	def __init__(self, asXML=None, verbose=None, output=None, silent=True, hostname=None, username=None, context=None, workspace=None):
 		super().__init__(asXML=asXML, verbose=verbose, output=output, silent=silent, hostname=hostname, username=username, context=context, workspace=workspace)
 		
 
+	#________________________________________________________________________________________________
 	@args.operation(help='get a list of workflows')
 	@args.parameter(name='workflow_id', help='the ID of workflow')
 	@args.parameter(name='state_id', short='s', help='state id', default='')
@@ -900,9 +1020,64 @@ class Task(STEP):
 		
 		return instances
 		
+
+	#________________________________________________________________________________________________
 	@args.operation(help='get workflow task by id')
 	@args.parameter(name='id', help='the ID of workflow task')
 	def get(self, id):
 		return super().get('%s/%s'%(self.base,id))
 		
 
+	#________________________________________________________________________________________________
+	@args.operation(help='claim workflow task by id')
+	@args.parameter(name='id', help='the ID of workflow task')
+	def claim(self, id):
+		return super().post('%s/%s/claim'%(self.base,id))
+		
+
+	#________________________________________________________________________________________________
+	@args.operation(help='get events for workflow task by id')
+	@args.parameter(name='id', help='the ID of workflow task')
+	def events(self, id):
+		return super().get('%s/%s/events'%(self.base,id))
+		
+
+	#________________________________________________________________________________________________
+	@args.operation(help='trigger events workflow task by id')
+	@args.parameter(name='id', help='the ID of workflow task')
+	@args.parameter(name='event_id', help='id of event transition')
+	@args.parameter(name='anonymous', short='a', help='anonymously')
+	@args.parameter(name='message', short='m', help='optional message for transition')
+	def trigger(self, id, event_id, anonymous=False, message=None):
+		
+		payload = dict(
+			event = dict(
+				id=event_id,
+				anonymous=anonymous
+			)
+		)
+
+		if message:
+			payload['message'] = message
+		
+		if self.verbose:
+			print(json.dumps(payload, indent='\t'))
+
+		body = json.dumps(payload)
+
+		headers = { 
+			'Content-Type' : 'application/json',
+			'Accept' : 'application/json',
+		}
+			
+		result = super().post('%s/%s/trigger-event'%(self.base, id), body=body, headers=headers)
+		return json.loads(result)
+		
+
+	#________________________________________________________________________________________________
+	@args.operation(help='release workflow task by id')
+	@args.parameter(name='id', help='the ID of workflow task')
+	def release(self, id):
+		return super().post('%s/%s/release'%(self.base,id))
+		
+	
